	 PAGE  80,132
	 TITLE 'JULIAN5 - Convert Dates'
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* * Author:	    S-Pratt					   * *
;* * Date-Written:  March 1989					   * *
;* * (C) Copyright 1989 by Steve Pratt				   * *
;* *								   * *
;* * This program converts dates, Calendar to Julian or Julian to  * *
;* * Calendar.	It also calculates the day of the week if desired  * *
;* * for any date between 01/01/1583 and 12/31/2200.  The month    * *
;* * name will be returned for any valid date entered.		   * *
;* * See PARMS for format of linkage parameters.		   * *
;* *								   * *
;* * ------------------------------------------------------------- * *
;* *								   * *
;* *	  There are approximately 365.25 days in a year, 29.5	   * *
;* * days in a lunar month (the interval between new moons) and    * *
;* * 12.33 lunar months in a year.  The calendar at present in	   * *
;* * use in the Western world ignores the lunar month, and is	   * *
;* * divided into 11 months of 30 or 31 days plus 1 month	   * *
;* * (February) which normally has only 28 days, with an extra one * *
;* * added every 4 years (the leap year), to make the average	   * *
;* * length of the year exactly 365.25 days.			   * *
;* *	  This calendar was established by Julius Ceasar in 46 BC  * *
;* * and leaves a difference of 11 minutes between the calendar    * *
;* * year and the real year.  By 1582 this discrepancy had	   * *
;* * accumulated to 10 days, and Pope Gregory XIII made a small    * *
;* * correction to the Julian calendar by requiring that only	   * *
;* * centuries divisible by 400 (rather than 4) should be leap	   * *
;* * years.							   * *
;* *	  To omit the 10 days which had erroneously accumulated    * *
;* * Pope Gregory decreed on October 4, 1582 that the next day	   * *
;* * would be October 15, thereby eliminating the days October 5,  * *
;* * through October 14, from existance.			   * *
;* *	  The Gregorian calendar is only discrepant by 26 seconds  * *
;* * per year, and will remain accurate for thousands of years.    * *
;* *								   * *
;* * ------------------------------------------------------------- * *
;* *								   * *
;* *	 For the purpose of this program the following terminology * *
;* * is used... 						   * *
;* *								   * *
;* *	 Calendar Date - Date of the form MM/DD/YEAR		   * *
;* *	 Julian Date   - Date of the form YEAR/DDD		   * *
;* *								   * *
;* *		    MM = Month (01-12)				   * *
;* *		    DD = Day of month (01-31)			   * *
;* *		  YEAR = Year (1583-2200)			   * *
;* *		   DDD = Day of year (001-366)			   * *
;* *								   * *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	
	 PAGE
CGROUP	 GROUP CSEG, DSEG
	 ASSUME CS:CGROUP, DS:CGROUP, ES:CGROUP
	
MVC	 MACRO DEST,SOURCE,LENGTH  ; Move memory to memory
	 PUSH  CX		   ;	dest in ES, source in DS
	 PUSH  SI
	 PUSH  DI
	 CLD
	 LEA   SI,SOURCE
	 LEA   DI,DEST
	 MOV   CX,LENGTH
	 CLI
	 REP   MOVSB
	 STI
	 POP   DI
	 POP   SI
	 POP   CX
	 ENDM
CMPL	 MACRO FLD1,FLD2,LENGTH    ; Compare memory with memory
	 PUSH  CX		   ;   FLD1 is addressed by ES
	 PUSH  DI		   ;   FLD2 is addressed by DS
	 PUSH  SI
	 PUSH  DS
	 PUSH  ES
	 LEA   SI,FLD1
	 LEA   DI,FLD2
	 MOV   CX,LENGTH
	 PUSH  DS		   ; XCHG DS,ES
	 PUSH  ES
	 POP   DS
	 POP   ES
	 CLD
	 CLI
	 REPE  CMPSB
	 STI
	 POP   ES
	 POP   DS
	 POP   SI
	 POP   DI
	 POP   CX
	 ENDM
M	 MACRO NUMBER,MULTIPLIER   ; double word multiply
	 PUSH  DI
	 PUSH  SI
	 LEA   DI,NUMBER
	 LEA   SI,MULTIPLIER
	 CALL  MULTIPLY
	 POP   SI
	 POP   DI
	 ENDM
D	 MACRO NUMBER,DIVISOR	   ; double word divide
	 PUSH  DI
	 PUSH  SI
	 LEA   DI,NUMBER
	 LEA   SI,DIVISOR
	 CALL  DIVIDE
	 POP   SI
	 POP   DI
	 ENDM
	 .SALL
	
	 PAGE
CSEG	 SEGMENT PUBLIC
	
	 PUBLIC JULIAN5
;---< Julian5 >------------------------------------------------------;
;--------------------------------------------------------------------;
JULIAN5  PROC  NEAR
	 PUSH  AX		   ; Save Registers
	 PUSH  BX		   ; ES:BX contains address of PARMS
	 PUSH  CX
	 PUSH  DX
	 PUSH  DI
	 PUSH  SI
	 PUSH  ES
	 PUSH  DS
	 PUSHF
	 PUSH  CS		   ; DS = CS
	 POP   DS
BEGIN:	 ORG   $
	 PUSH  BX		   ; Make sure parms are all caps
	 MOV   SI,BX
	 LEA   BX,UPPERC
	 MOV   CX,13
BEG02:	 MOV   AL,ES:[SI]
	 XLAT  UPPERC
	 MOV   ES:[SI],AL
	 INC   SI
	 LOOP  BEG02
	 POP   BX
	 MOV   CX,51		   ; Move blanks to PLEAPYR,
	 LEA   DI,[BX].PLEAPYR	   ;   PDAY, PERRCODE,PERRMSG
	 MOV   AL,20H
	 CLD
	 CLI
	 REP   STOSB
	 STI
	 CMPL  [BX].PYEAR,C_1583,4 ; Date must be in proper range
	 JB    BEG10		   ;   for conversion
	 CMPL  [BX].PYEAR,C_2200,4 ;   1/1/1583 - 12/31/2200
	 JNA   BEG20
BEG10:	 MVC   [BX].PERRCODE,ERR8,32 ; otherwise return with error
	 JMP   GOBACK
BEG20:	 CMP   BYTE PTR ES:[BX].PCODE,'C' ; If code is equal to 'C'
	 JNE   BEG30		   ;	perform TOJUL
	 CALL  TOJUL
	 JMP   BEG50
BEG30:	 CMP   BYTE PTR ES:[BX].PCODE,'J' ; Else if code is equal to 'J'
	 JNE   BEG40		   ;	perform TOCAL
	 CALL  TOCAL
	 JMP   BEG50
BEG40:	 MVC   [BX].PERRCODE,ERR1,32 ; Else move error-message
	 JMP   GOBACK		   ;	and return
BEG50:	 CMP   BYTE PTR ES:[BX].PERRCODE,20H ; If any errors
	 JNE   GOBACK		   ;	return
	 CMP   BYTE PTR ES:[BX].POPTION,'D' ; Else if option is 'D'
	 JNE   BEG60
	 CALL  DAY		   ;	perform DAY
	 JMP   BEG070
BEG60:	 CMP   BYTE PTR ES:[BX].POPTION,20H ; Else if option is blank
	 JE    BEG070		   ;	return
	 MVC   [BX].PERRCODE,ERR4,32 ; Else move error-message
	 JMP   GOBACK
BEG070:  MOV   AX,WMM
	 DEC   AX
	 MUL   H_9
	 MOV   DI,AX
	 MVC   [BX].PMONTH,MONTHTABLE[DI],9
GOBACK:  POPF			   ; Restore registers
	 POP   DS
	 POP   ES
	 POP   SI
	 POP   DI
	 POP   DX
	 POP   CX
	 POP   BX
	 POP   AX
	 RET			   ; Return to calling program
JULIAN5  ENDP
	
;---< To Jul >-------------------------------------------------------;
; Convert Calendar date to Julian date. 			     ;
;								     ;
;	 Algorithm: 1.	DDD = DD + (((MM+2) * 3055) / 100) - 91      ;
;		    2.	If DDD > (59 + LY) THEN DDD = DDD - 2 + LY   ;
;								     ;
;	 Where: MM = Month (1-12), DD = Day of month (1-31)	     ;
;		LY = 1 if leap year, 0 otherwise		     ;
;		DDD = Julian day of year (1-366)		     ;
;								     ;
;--------------------------------------------------------------------;
TOJUL	 PROC  NEAR		   ; --- Calendar to Julian ---
	 LEA   SI,[BX].PCMM	   ; If PCALDATE is not
	 MOV   CX,8
TOJ04:	 MOV   AL,ES:[SI]	   ;   numeric
	 CMP   AL,30H
	 JB    TOJ06
	 CMP   AL,39H
	 JA    TOJ06
	 INC   SI
	 LOOP  TOJ04
	 JMP   TOJ10
TOJ06:	 MVC   [BX].PERRCODE,ERR2,32 ; move error message
	 JMP   TOJEXIT		   ;   and exit
TOJ10:	 CMPL  [BX].PCMM,C_00,2    ; If month is equal to zero
	 JE    TOJ20		   ;   then error
	 CMPL  [BX].PCMM,C_13,2    ; If month is greater than 12
	 JB    TOJ30
TOJ20:	 MVC   [BX].PERRCODE,ERR3,32 ;	 move error message
	 JMP   TOJEXIT		   ;	 and exit
TOJ30:	 CMPL  [BX].PCDD,C_00,2    ; else if day is equal to zero
	 JE    TOJ40		   ;   then error
	 CMPL  [BX].PCDD,C_32,2    ; else if day is greater than 31
	 JB    TOJ50
TOJ40:	 MVC   [BX].PERRCODE,ERR3,32 ; move error message
	 JMP   TOJEXIT		   ;   and exit
TOJ50:	 LEA   DI,[BX].PCMM
	 MOV   CX,2
	 CALL  TO_BINRY
	 MOV   WMM,AX
	 LEA   DI,[BX].PCDD	   ; Well let's do some
	 MOV   CX,2		   ;   calculations now
	 CALL  TO_BINRY
	 MOV   WDD,AX
	 LEA   DI,[BX].PYEAR
	 MOV   CX,4
	 CALL  TO_BINRY
	 MOV   WYEAR,AX
	 MOV   AX,WMM		   ; DDD = DD + (((MM+2)
	 ADD   AX,+2		   ;	  * 3055) / 100) - 91
	 MUL   F_3055
	 DIV   F_100
	 ADD   AX,WDD
	 SUB   AX,+91		   ; AX contains DDD
	 CALL  LEAPYR		   ; perform LEAPYR
	 CALL  MAXDAYS		   ; perform MAXDAYS
	 CMP   BYTE PTR ES:[BX].PERRCODE,20H ; If any errors
	 JNE   TOJEXIT		   ;	       exit
	 MOV   DX,LY		   ; If DDD is greater than
	 ADD   DX,+59		   ;   (59 + LY)  compute
	 CMP   AX,DX		   ;	  DDD = DDD - 2 + LY
	 JNA   TOJ60
	 SUB   AX,+2
	 ADD   AX,LY
TOJ60:	 MOV   WDDD,AX
	 PUSH  BX
	 LEA   DI,[BX].PJDDD	   ; I hope this is correct
	 AND   DX,0
	 AND   BX,0
	 MOV   CX,3
	 CALL  TO_ASCII
	 POP   BX
TOJEXIT: RET
TOJUL	 ENDP
	
;---< To Cal >-------------------------------------------------------;
; Convert Julian date to Calendar date. 			     ;
;								     ;
;	 Algorithm: 1. If DDD > (59 + LY) then DDD = DDD + 2 - LY    ;
;		    2. MM = ((DDD + 91) * 100) / 3055		     ;
;		    3. DD = (DDD + 91) - ((MM * 3055) / 100)	     ;
;		    4. MM = MM - 2				     ;
;								     ;
;	 Where: DDD = Julian day of year (1-366)		     ;
;		MM = Month (1-12), DD = Day of month (1-31)	     ;
;		LY = 1 if leap year, 0 if not			     ;
;								     ;
;--------------------------------------------------------------------;
TOCAL	 PROC  NEAR		   ; --- Julian to Calendar ---
	 LEA   SI,[BX].PYEAR	   ; If PJULDATE is not
	 MOV   CX,7
TOC04:	 MOV   AL,ES:[SI]	   ;   numeric
	 CMP   AL,30H
	 JB    TOC06
	 CMP   AL,39H
	 JA    TOC06
	 INC   SI
	 LOOP  TOC04
	 JMP   TOC10
TOC06:	 MVC   ES:[BX].PERRCODE,ERR5,32 ; move error message
	 JMP   TOCEXIT		   ;	  and exit
TOC10:	 CMPL  ES:[BX].PJDDD,C_000,3 ; If DDD is equal to zero
	 JNE   TOC20
	 MVC   ES:[BX].PERRCODE,ERR3,32 ; then error
	 JMP   TOCEXIT
TOC20:	 CMPL  [BX].PJDDD,C_367,3 ; If DDD is greater than 366
	 JB    TOC30
	 MVC   [BX].PERRCODE,ERR7,32 ; move error message
	 JMP   TOCEXIT		   ;   and exit
TOC30:	 LEA   DI,[BX].PJDDD	   ; Time to start the calculations
	 MOV   CX,3
	 CALL  TO_BINRY
	 MOV   WDDD,AX
	 LEA   DI,[BX].PYEAR
	 MOV   CX,4
	 CALL  TO_BINRY
	 MOV   WYEAR,AX
	 CALL  LEAPYR		   ; Perform LEAPYR
	 CMP   BYTE PTR ES:[BX].PLEAPYR,'N' ; If not a leap year
	 JNE   TOC40		   ;   and DDD is greater than
	 CMPL  [BX].PJDDD,C_366,3  ;	365
	 JB    TOC40
	 MVC   [BX].PERRCODE,ERR7,32 ; move error message
	 JMP   TOCEXIT		   ;   and exit
TOC40:	 MOV   AX,WDDD
	 MOV   DX,LY		   ; If DDD is greater than (59 + LY)
	 ADD   DX,+59
	 CMP   AX,DX		   ;   compute
	 JNA   TOC50		   ;   DDD = DDD + 2 - LY
	 ADD   AX,+2
	 SUB   AX,LY
TOC50:	 ADD   AX,+91		   ; MM = ((DDD + 91) * 100) / 3055
	 MOV   WORK,AX
	 MUL   F_100
	 DIV   F_3055
	 MOV   WMM,AX
	 MUL   F_3055		   ; DD = (DDD+91) - ((MM * 3055) / 100)
	 DIV   F_100
	 MOV   DX,WORK
	 SUB   DX,AX
	 MOV   WDD,DX
	 DEC   WMM		   ; MM = MM - 2
	 DEC   WMM
	 PUSH  BX
	 LEA   DI,[BX].PCDD
	 MOV   AX,WDD
	 AND   DX,0		   ; Here is the final
	 AND   BX,0		   ;  result
	 MOV   CX,2
	 CALL  TO_ASCII
	 POP   BX
	 PUSH  BX
	 MOV   AX,WMM
	 LEA   DI,[BX].PCMM
	 AND   BX,0
	 MOV   CX,2
	 CALL  TO_ASCII
	 POP   BX
TOCEXIT: RET			   ; Exit
TOCAL	 ENDP
	
;---< Leapyr >-------------------------------------------------------;
; Decide if the year is a leap year.				     ;
;								     ;
;	 On entry: WYEAR = year 				     ;
;								     ;
;	 On exit:  PLEAPYR = 'Y' if leap year, 'N' if not	     ;
;		   LY = 1 if leap year, 0 if not		     ;
;--------------------------------------------------------------------;
LEAPYR	 PROC  NEAR		   ; --- Leap year check ---
	 PUSH  AX
	 MOV   AX,WYEAR
	 MOV   CX,AX
	 XOR   DX,DX
	 DIV   F_4		   ; If YEAR = ((YEAR / 4) * 4)
	 MUL   F_4		   ;	 this could be a leap year
	 CMP   AX,CX
	 JNE   LEA20
	 DIV   F_100		   ; Is this the start of a century?
	 MUL   F_100
	 CMP   AX,CX
	 JNE   LEA10
	 DIV   F_400		   ;   yes, then it must be evenly
	 MUL   F_400		   ;   divisible by 400 to be a leap year
	 CMP   AX,CX
	 JNE   LEA20
LEA10:	 MOV   WORD PTR LY,1	   ; Move 1 to LY
	 MOV   BYTE PTR ES:[BX].PLEAPYR,'Y'
	 JMP   LEAEXIT
LEA20:	 MOV   WORD PTR LY,0	   ; Move 0 to LY
	 MOV   BYTE PTR ES:[BX].PLEAPYR,'N'
LEAEXIT: POP   AX
	 RET			   ; Exit
LEAPYR	 ENDP
	
	
;---< Maxdays >------------------------------------------------------;
; Decide if WDD contains a proper value for the given month (WMM)    ;
;--------------------------------------------------------------------;
MAXDAYS  PROC  NEAR		   ; --- Max. No. of Days Per Month ---
	 PUSH  AX		   ;		Algorithm
	 MOV   AX,WMM
	 MOV   DX,WMM
	 MOV   CL,3
	 SAR   DX,CL		   ; Exclusive OR the
	 XOR   DX,AX		   ;   most significant bit with
	 TEST  DX,1		   ;   the least significant bit
	 JZ    MAX10		   ; If the result is 1
	 MOV   AX,31		   ;   the number of days is 31
	 JMP   MAXTEST
MAX10:	 CMP   AX,2		   ; If the month is 2
	 JNE   MAX30		   ;   the number of days
	 MOV   AX,28		   ;   is 28 or 29
	 ADD   AX,LY
	 JMP   MAXTEST
MAX30:	 MOV   AX,30		   ; Otherwise no. of days is 30
MAXTEST: MOV   DX,WDD		   ; Test for maximum days
	 CMP   DX,AX		   ;   for the month
	 JNA   MAXEXIT
	 MVC   [BX].PERRCODE,ERR6,32
MAXEXIT: POP   AX
	 RET			   ; Exit
MAXDAYS  ENDP
	
;--------------------------------------------------------------------;
;    The day-of-week algorithm is ...				     ;
;								     ;
;	 Y = Year, D = Julian day, X = Day of week		     ;
;								     ;
;	       1. X = ((X-1) * 365.25) + D			     ;
;	       2.      If Y > 2100, X = X-1			     ;
;		  Else if Y < 1701, X = X+3			     ;
;		  Else if Y < 1801, X = X+2			     ;
;		  Else if Y < 1901, X = X+1			     ;
;	       3. X = ((Intiger X) Mod 7) - 1			     ;
;	       4. If X < 0, X = 6				     ;
;	       5. X is day of week,				     ;
;		       Sunday=0, Monday=1, etc. 		     ;
;								     ;
;    It is interesting to note that the same day of the week	     ;
; occurs on the same date every 28 years.			     ;
;--------------------------------------------------------------------;
	
DAY	 PROC  NEAR		   ; --- Day of Week Algorithm ---
	 MOV   R5,0
	 MOV   R7,0
	 MOV   AX,WYEAR 	   ; R4 & R6 = (YEAR - 1)
	 DEC   AX
	 MOV   R4,AX
	 MOV   R6,AX
	 MUL   F_365		   ; R4:R5 = R4 * 365
	 ADD   AX,WDDD		   ; R4:R5 = (R4:R5 + DDD)
	 ADC   DX,0
	 MOV   R4,DX
	 MOV   R5,AX
	 MOV   AX,R6		   ; R6:R7 = (YEAR - 1) * 25
	 MUL   F_25
	 MOV   R6,DX
	 MOV   R7,AX
	 M     R4,F_100 	   ; R6:R7 = (R4:R5 * 100) + R6:R7
	 MOV   AX,R5
	 MOV   DX,R4
	 ADD   R7,AX
	 ADC   R6,DX
	 D     R6,F_100 	   ; R6:R7 = R6:R7 / 100
	 CMP   WORD PTR WYEAR,+2100 ; Correct for the century
	 JNA   DAY10		    ;	as per Pope Gregory XIII
	 SUB   R7,1
	 SBB   R6,0
	 JMP   DAY20
DAY10:	 CMP   WORD PTR WYEAR,+1900
	 JA    DAY20
	 ADD   R7,1
	 ADC   R6,0
	 CMP   WORD PTR WYEAR,+1800
	 JA    DAY20
	 ADD   R7,1
	 ADC   R6,0
	 CMP   WORD PTR WYEAR,+1700
	 JA    DAY20
	 ADD   R7,1
	 ADC   R6,0
DAY20:	 D     R6,F_7		   ; R6:R7 Rem. DX = R6:R7 / 7
	 SUB   DX,+1		   ; DX = DX - 1
	 JNL   DAY30		   ; If DX < 0
	 MOV   DX,6		   ;   Move 6 to DX
DAY30:	 MOV   AX,DX		   ; Calculate displacement
	 MUL   H_9
	 MOV   DI,AX
	 MVC   [BX].PDAY,DAYTABLE[DI],9 ; and move day from table
	 RET
DAY	 ENDP
	
;---< TO_ASCII >-----------------------------------------------------;
; On Entry : DX:AX Contains number to be converted		     ;
;	     BL    Contains 'Z' to zero suppress		     ;
;	     ES:DI Contains offset of field which will contain	     ;
;		     the converted string			     ;
;	     CX    is the length of this field, max. 10 	     ;
;--------------------------------------------------------------------;
TO_ASCII PROC  NEAR
	 PUSH  AX
	 PUSH  BX
	 PUSH  CX
	 PUSH  DX
	 PUSH  SI
	 PUSH  DI
	 MOV   BH,'0'
	 CMP   BL,'Z'
	 JNE   TO_A0
	 MOV   BH,20H
TO_A0:	 PUSH  CX		   ; length of destination field
	 PUSH  DI		   ; Save destination pointer
TO_A1:	 MOV   BYTE PTR ES:[DI],BH ; Initialize output
	 INC   DI
	 LOOP  TO_A1
	 POP   DI		   ; Restore destination pointer
	 POP   CX
	 ADD   DI,CX		   ; point to end of buffer
	 DEC   DI
	 MOV   WORD PTR BINARY+2,DX ; Store number to convert
	 MOV   WORD PTR BINARY,AX
	 LEA   SI,BINARY
TO_A2:	 PUSH  CX		   ; save destination length
	 PUSH  SI		   ; save source pointer
	 AND   BX,0		   ; done flag
	 MOV   CX,2		   ; 2 words long
	 AND   DX,0		   ; previous remainder
	 ADD   SI,2		   ; point to last word
TO_A3:	 PUSH  CX		   ; save count
	 MOV   AX,[SI]		   ; get 16 bit digit
	 MOV   CX,10		   ; divisor of 10
	 DIV   CX
	 MOV   [SI],AX		   ; put 16 bit digit back
	 OR    BX,AX		   ; check for zero
	 SUB   SI,2		   ; point to next 16 bit digit
	 POP   CX
	 LOOP  TO_A3
	 ADD   DL,30H		   ; convert digit to decimal
	 MOV   ES:[DI],DL	   ; and store it
	 DEC   DI
	 POP   SI
	 POP   CX
	 DEC   CX
	 JCXZ  TO_A4		   ; exit if field is full
	 CMP   BX,0		   ; done?
	 JNZ   TO_A2
TO_A4:	 POP   DI
	 POP   SI
	 POP   DX
	 POP   CX
	 POP   BX
	 POP   AX
	 RET			   ; And return
TO_ASCII ENDP
	
;---< To_Binry >-----------------------------------------------------;
; On entry:    ES:DI contain address of ASCII string		     ;
;		  CX contains length of string			     ;
; On exit:     DX:AX contains the converted binary number	     ;
;--------------------------------------------------------------------;
TO_BINRY PROC  NEAR
	 PUSH  DI
	 PUSH  CX
	 PUSH  SI
	 AND   AX,0		   ; Result is zero to start
	 AND   DX,0
TO_B10:  CMP   BYTE PTR ES:[DI],20H ; Scan past leading blanks
	 JNE   TO_B30
	 INC   DI
	 LOOP  TO_B10
	 JMP   TO_B50		   ; Exit if all spaces
TO_B30:  PUSH  CX		   ; save loop counter
	 MOV   SI,10		   ;	 multiplier
	 MOV   CX,DX		   ; save DX in CX
	 MUL   SI		   ; multiply AX by 10
	 PUSH  AX		   ; save result
	 PUSH  DX
	 MOV   AX,CX		   ; now multiply DX by 10
	 MUL   SI
	 JNC   TO_B36
	 POP   DX		   ; too big
	 POP   AX		   ;   then split
	 JMP   TO_B40
TO_B36:  MOV   DX,AX
	 POP   AX		   ; add DX for new high order word
	 ADD   DX,AX
	 POP   AX		   ; and restore low order word
	 JC    TO_B40
	 MOV   CL,BYTE PTR ES:[DI] ; Fetch ASCII code
	 AND   CX,000FH 	   ;	 Save only low order bits
	 ADD   AX,CX		   ;	 and update partial result
	 ADC   DX,0
	 JC    TO_B40		   ; Exit if result is too big
	 POP   CX		   ;   restore high order word
	 INC   DI		   ;	 otherwise increment DI
	 LOOP  TO_B30		   ;	 and continue
	 JMP   TO_B50
TO_B40:  POP   CX
	 MOV   AX,0FFFFH	   ; Too big
	 MOV   DX,0FFFFH	   ;	 then max it out
TO_B50:  POP   SI
	 POP   CX
	 POP   DI
	 RET
TO_BINRY ENDP
	
;---< Multiply >-----------------------------------------------------;
; Multiply a double word number by a single word multiplier.	     ;
;	 On entry:  DS:DI points to first number		     ;
;		    DS:SI points to multiplier			     ;
;		    WORK  double word work area 		     ;
;	 On exit:   DS:DI is replaced by double word result	     ;
;--------------------------------------------------------------------;
MULTIPLY PROC  NEAR
	 PUSH  AX
	 PUSH  DX
	 PUSH  ES
	 MOV   AX,[DI+2]	   ; Multiply
	 MUL   WORD PTR [SI]
	 MOV   WORD PTR WORK,DX
	 MOV   WORD PTR WORK+2,AX
	 MOV   AX,[DI]
	 MUL   WORD PTR [SI]
	 ADD   WORK,AX
	 MOV   DX,WORK		   ; move result
	 MOV   [DI],DX
	 MOV   AX,WORK+2
	 MOV   [DI+2],AX
	 POP   ES
	 POP   DX
	 POP   AX
	 RET
MULTIPLY ENDP
	
;---< Divide >-------------------------------------------------------;
; Divide a double word number by a single word divisor. 	     ;
;	 On entry:  DS:DI points to first number		     ;
;		    DS:SI points to divisor			     ;
;	 On exit:   DS:DI is replaced by double word result	     ;
;		    DX	  contains remainder			     ;
;--------------------------------------------------------------------;
DIVIDE	 PROC  NEAR
	 PUSH  AX
	 AND   DX,0
	 MOV   AX,[DI]
	 DIV   WORD PTR [SI]
	 MOV   [DI],AX
	 MOV   AX,[DI+2]
	 DIV   WORD PTR [SI]
	 MOV   [DI+2],AX
	 POP   AX
	 RET
DIVIDE	 ENDP
	
CSEG	 ENDS
	
	 PAGE
DSEG	 SEGMENT PUBLIC
	
DAYTABLE EQU   $
	 DB    'Sunday   '
	 DB    'Monday   '
	 DB    'Tuesday  '
	 DB    'Wednesday'
	 DB    'Thursday '
	 DB    'Friday   '
	 DB    'Saturday '
	
MONTHTABLE EQU $
	 DB    'January  '
	 DB    'February '
	 DB    'March    '
	 DB    'April    '
	 DB    'May      '
	 DB    'June     '
	 DB    'July     '
	 DB    'August   '
	 DB    'September'
	 DB    'October  '
	 DB    'November '
	 DB    'December '
	
ERR1	 DB    '1Code invalid - should be C or J'
ERR2	 DB    '2Calendar date is not numeric   '
ERR3	 DB    '3Invalid date - not converted   '
ERR4	 DB    '4Option not blank or D          '
ERR5	 DB    '5Julian date is not numeric     '
ERR6	 DB    '6Too many days for this month   '
ERR7	 DB    '7Too many days for this year    '
ERR8	 DB    '8Not in range 1583/001, 2200/365'
	
R4	 DW    0		   ; Work registers for DAY
R5	 DW    0
R6	 DW    0
R7	 DW    0
	
WORK	 DD    ?
WMM	 DW    ?
WDD	 DW    ?
WYEAR	 DW    ?
WDDD	 DW    ?
LY	 DW    ?		   ; used by LEAPYR
BINARY	 DB    4 DUP(0) 	   ; Work area for TO_ASCII
	
C_1583	 DB    '1583'		   ; Literal pool
C_2200	 DB    '2200'
C_000	 DB    '000'
C_366	 DB    '366'
C_367	 DB    '367'
C_00	 DB    '00'
C_13	 DB    '13'
C_32	 DB    '32'
F_3055	 DW    +3055
F_400	 DW    +400
F_365	 DW    +365
F_100	 DW    +100
F_25	 DW    +25
F_7	 DW    +7
F_4	 DW    +4
H_9	 DB    +9
	
UPPERC	 DB    00H,01H,02H,03H,04H,05H,06H,07H ; translate table for
	 DB    08H,09H,0AH,0BH,0CH,0DH,0EH,0FH ;   upper case alpha
	 DB    10H,11H,12H,13H,14H,15H,16H,17H
	 DB    18H,19H,1AH,1BH,1CH,1DH,1EH,1FH
	 DB    20H,21H,22H,23H,24H,25H,26H,27H
	 DB    28H,29H,2AH,2BH,2CH,2DH,2EH,2FH
	 DB    30H,31H,32H,33H,34H,35H,36H,37H
	 DB    38H,39H,3AH,3BH,3CH,3DH,3EH,3FH
	 DB    40H,41H,42H,43H,44H,45H,46H,47H
	 DB    48H,49H,4AH,4BH,4CH,4DH,4EH,4FH
	 DB    50H,51H,52H,53H,54H,55H,56H,57H
	 DB    58H,59H,5AH,5BH,5CH,5DH,5EH,5FH
	 DB    60H,41H,42H,43H,44H,45H,46H,47H
	 DB    48H,49H,4AH,4BH,4CH,4DH,4EH,4FH
	 DB    50H,51H,52H,53H,54H,55H,56H,57H
	 DB    58H,59H,5AH,7BH,7CH,7DH,7EH,7FH
	 DB    80H,81H,82H,83H,84H,85H,86H,87H
	 DB    88H,89H,8AH,8BH,8CH,8DH,8EH,8FH
	 DB    90H,91H,92H,93H,94H,95H,96H,97H
	 DB    98H,99H,9AH,9BH,9CH,9DH,9EH,9FH
	 DB    0A0H,0A1H,0A2H,0A3H,0A4H,0A5H,0A6H,0A7H
	 DB    0A8H,0A9H,0AAH,0ABH,0ACH,0ADH,0AEH,0AFH
	 DB    0B0H,0B1H,0B2H,0B3H,0B4H,0B5H,0B6H,0B7H
	 DB    0B8H,0B9H,0BAH,0BBH,0BCH,0BDH,0BEH,0BFH
	 DB    0C0H,0C1H,0C2H,0C3H,0C4H,0C5H,0C6H,0C7H
	 DB    0C8H,0C9H,0CAH,0CBH,0CCH,0CDH,0CEH,0CFH
	 DB    0D0H,0D1H,0D2H,0D3H,0D4H,0D5H,0D6H,0D7H
	 DB    0D8H,0D9H,0DAH,0DBH,0DCH,0DDH,0DEH,0DFH
	 DB    0E0H,0E1H,0E2H,0E3H,0E4H,0E5H,0E6H,0E7H
	 DB    0E8H,0E9H,0EAH,0EBH,0ECH,0EDH,0EEH,0EFH
	 DB    0F0H,0F1H,0F2H,0F3H,0F4H,0F5H,0F6H,0F7H
	 DB    0F8H,0F9H,0FAH,0FBH,0FCH,0FDH,0FEH,0FFH
	
DSEG	 ENDS
	
	 PAGE
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* *		    This is the Linkage Section 		   * *
;* *								   * *
;* * The calling program should mirror this.  Required is a date   * *
;* * (Julian or Calendar) and a code (C or J).			   * *
;* *	 C = Convert Calendar to Julian 			   * *
;* *	 J = Convert Julian to Calendar 			   * *
;* *	Option must be blank or D				   * *
;* *	 D = Compute Day of Week				   * *
;* * The date must fall within the range 1583/001 - 2200/365.	   * *
;* *								   * *
;* * The address of this area must be passed in ES:BX		   * *
;* *								   * *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	
	
PARMS	 STRUC
; PCALDATE			   ; Calendar date
PCMM	 DB    2 DUP(?)
PCDD	 DB    2 DUP(?)
; PJULDATE			   ; Julian date
PYEAR	 DB    4 DUP(?)
PJDDD	 DB    3 DUP(?)
PCODE	 DB    ?		   ; Code, C = ToJul, J = ToCal
POPTION  DB    ?		   ; Option, D = Calc Day of Week
PLEAPYR  DB    ?		   ; Y = Leap year, N = Not
PMONTH	 DB    9 DUP(?) 	   ; Month name
PDAY	 DB    9 DUP(?) 	   ; Day of Week
PERRCODE DB    ?		   ; Error number
PERRMSG  DB    31 DUP(?)	   ; Error message
PARMS	 ENDS
	
	 END
